//??unsure if I really should support multiple callbacks attached to a single event, in most cases 1 would suffice
// ****************************************************************************************
//
// Copyright (C) 2011 Jonathan Sandusky
//
// This software is distributed under the terms of the Eclipse Public License v1.0.
// A copy of the license may be obtained at: http://www.eclipse.org/legal/epl-v10.html
//
// *************************************************************************************************


#include "CallbackComponent.h"

#include "CallbackManager.h"

#include <GameEngine/GameEvent.h>
#include <GameEngine\GameModules.h>
#include <GameEngine/GameEntity.h>
#include <GameEngine/EventManager.h>
#include <GameEngine/GameLog.h>
#include <XMLParser\utXMLParser.h>

#include <sstream>

GameComponent* CallbackComponent::createComponent( GameEntity* owner )
{
	return new CallbackComponent( owner );
}

CallbackComponent::CallbackComponent(GameEntity *owner) : GameComponent(owner, "CallbackHandler")
{
	CallbackManager::instance()->addComponent(this);	
}

CallbackComponent::~CallbackComponent()
{
	CallbackManager::instance()->removeComponent(this);
}

void CallbackComponent::executeEvent(GameEvent *event)
{
	iterator it = callbacks_.find(event->id());
	if (it != callbacks_.end()) {
		for (size_t i = 0; i < it->second.size(); ++i) {
			(*it->second[i].call_)(false, *event);
		}
	}
}

void CallbackComponent::loadFromXml(const XMLNode *description)
{
	const int childCount = description->nChildNode();
	for (int i = 0; i < childCount; ++i) {
		XMLNode node = description->getChildNode(i);
		if (node.getName() == "Func") {
			std::string func = node.getAttribute("name","");
			GameEvent::EventID e = GameModules::eventRegistry()->getEventID(node.getAttribute("event",""));
			if (func.length() && CallbackManager::instance()->factory() && e != GameEvent::E_INVALID) {
				CALL c = CallbackManager::instance()->restoreCallback(func.c_str());
				if (c != 0x0) {
					CallInfo info;
					info.call_ = c; info.name_ = func;
					callbacks_[e].push_back(info);
				} else {
					std::stringstream ss;
					ss << "Unable to find function: " << func;
					GameLog::errorMessage(ss.str().c_str());
				}
			} else {
				std::stringstream ss;
				ss << "Unable to establish a satisfactory environment for restoring function: " << func;
				GameLog::errorMessage(ss.str().c_str());
			}
		}
	}
}

bool CallbackComponent::checkEvent(GameEvent* event)
{
	iterator it = callbacks_.find(event->id());
	if (it != callbacks_.end()) {
		bool ret = true;
		for (size_t i = 0; i < it->second.size(); ++i)
			ret &= (*(it->second[i].call_))(true, *event);
		return ret;
	}
	return true;
}

void CallbackComponent::addCallback(GameEvent::EventID id, CALL aCall, const char* aName) 
{
	CallInfo info;
	info.call_ = aCall;
	info.name_ = aName;
	callbacks_[id].push_back(info);
}

bool CallbackComponent::removeCallback(GameEvent::EventID id, CALL aCall)
{
	iterator it = callbacks_.find(id);
	bool ret = false;
	if (it != callbacks_.end()) {
		for (std::vector<CallInfo>::iterator callit = it->second.begin(); callit != it->second.end(); ++callit) {
			if (callit->call_ == aCall) {
				it->second.erase(callit);
				callit = it->second.begin(); //sigh
				ret = true;
			}
		}
	}
	return ret;
}