// ****************************************************************************************
//
// Copyright (C) 2011 Jonathan Sandusky
// 
// This software is distributed under the terms of the Eclipse Public License v1.0.
// A copy of the license may be obtained at: http://www.eclipse.org/legal/epl-v10.html
//
// *************************************************************************************************
//


#ifndef CallbackComponent_H_
#define CallbackComponent_H_

#include <GameEngine/GameComponent.h>

/**
 * Component to execute callbacks during events
 */
typedef bool (*CALL)(bool, GameEvent&);

class CallbackComponent : public GameComponent
{
public:
	static GameComponent* createComponent( GameEntity* owner );
	CallbackComponent(GameEntity* owner);
	~CallbackComponent();


	bool checkEvent(GameEvent* event);
	void executeEvent(GameEvent* event);

	void loadFromXml(const XMLNode* description);

	void addCallback(GameEvent::EventID id, CALL aCall, const char* aName);
	bool removeCallback(GameEvent::EventID id, CALL aCall);

private:
	struct CallInfo {
		CALL call_;
		std::string name_; //to persist or init
	};
	typedef std::map<GameEvent::EventID, std::vector<CallInfo> >::iterator iterator;
	std::map<GameEvent::EventID, std::vector<CallInfo> > callbacks_;

};
#endif 

