// ****************************************************************************************
//
// Copyright (C) 2011 Jonathan Sandusky
// 
// This software is distributed under the terms of the Eclipse Public License v1.0.
// A copy of the license may be obtained at: http://www.eclipse.org/legal/epl-v10.html
//
// *************************************************************************************************


#include "GameEngine_Callback.h"

#include <GameEngine/GameEntity.h>
#include <GameEngine/GameWorld.h>
#include <GameEngine/GameModules.h>

#include "CallbackManager.h"
#include "CallbackComponent.h"

namespace GameEngine
{
	CALLBACKAPI void registerCallback(unsigned int aEntityID, unsigned short aEventID, CALLBACK_FUNC aFunc, const char* aName /*for restoring*/)
	{
		CallbackComponent* component = 0;
		GameEntity* entity = GameModules::gameWorld()->entity(aEntityID);
		if( entity && ( component = static_cast<CallbackComponent*>(entity->component("CallbackHandler")) ) != 0 ) {
			return component->addCallback((GameEvent::EventID)aEventID, aFunc, aName);
		}
	}

	CALLBACKAPI bool removeCallback(unsigned int aEntityID, unsigned short aEventID, CALLBACK_FUNC aFunc)
	{		
		CallbackComponent* component = 0;
		GameEntity* entity = GameModules::gameWorld()->entity(aEntityID);
		if( entity && ( component = static_cast<CallbackComponent*>(entity->component("CallbackHandler")) ) != 0 )
		{
			component->removeCallback((GameEvent::EventID)aEventID, aFunc);
		}	
		return 0;
	}

	CALLBACKAPI void registerFactoryFunc(FACTORY_FUNC aFunc) {
		CallbackManager::instance()->setFactory(aFunc);
	}
}

